'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = transformAssets;

var _path = require('path');

var _fs = require('fs');

var _cssParse = require('css-parse');

var _cssParse2 = _interopRequireDefault(_cssParse);

var _toCamelCase = require('to-camel-case');

var _toCamelCase2 = _interopRequireDefault(_toCamelCase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultOptions = {};
var RULE = 'rule';
var DECLARATION = 'declaration';
var styleSheetConstants = {
  'hairline-width': 'require(\'react-native\').StyleSheet.hairlineWidth'
};

function normalizeValue(property, value) {
  var normalizeProps = ['flex', 'width', 'height', 'top', 'right', 'bottom', 'left', 'font-size', 'line-height', 'margin-top', 'margin-right', 'margin-bottom', 'margin-left', 'padding-top', 'padding-right', 'padding-bottom', 'padding-left', 'padding-vertical', 'padding-horizontal', 'border-top-width', 'border-right-width', 'border-bottom-width', 'border-left-width', 'border-radius', 'border-top-left-radius', 'border-top-right-radius', 'border-bottom-left-radius', 'border-bottom-right-radius', 'border-width', 'shadow-opacity', 'shadow-radius', 'elevation'];

  if (normalizeProps.indexOf(property) > -1) {

    var camelizedPropName = (0, _toCamelCase2.default)(property);

    if (styleSheetConstants[value]) {
      return _defineProperty({}, camelizedPropName, value);
    }

    var v = parseInt(value);

    if (isNaN(v)) {
      throw new Error('Unexpected string value ' + value + ' for ' + property + '.');
    }

    return _defineProperty({}, camelizedPropName, v);
  }

  return {};
}

function expandProperty(property, value) {
  var expandProps = ['margin', 'padding', 'shadow-offset'];

  if (expandProps.indexOf(property) > -1) {
    var _shorthand;

    var shorthandValues = value.split(' ').map(function (n) {
      return parseInt(n, 10);
    });

    var _shorthandValues = _slicedToArray(shorthandValues, 4);

    var v1 = _shorthandValues[0];
    var v2 = _shorthandValues[1];
    var v3 = _shorthandValues[2];
    var v4 = _shorthandValues[3];

    var len = shorthandValues.length;

    if (property === 'shadow-offset') {
      return {
        'shadowOffset': {
          width: v1,
          height: v2
        }
      };
    }

    if (len === 1) {
      return _defineProperty({}, '' + property, v1);
    }

    var shorthand = (_shorthand = {}, _defineProperty(_shorthand, '' + property, 0), _defineProperty(_shorthand, property + 'Top', v1), _defineProperty(_shorthand, property + 'Right', v2), _shorthand);

    if (len === 2) {
      var _extends2;

      return _extends({}, shorthand, (_extends2 = {}, _defineProperty(_extends2, property + 'Bottom', v1), _defineProperty(_extends2, property + 'Left', v2), _extends2));
    }

    if (len === 3) {
      var _extends3;

      return _extends({}, shorthand, (_extends3 = {}, _defineProperty(_extends3, property + 'Bottom', v3), _defineProperty(_extends3, property + 'Left', v2), _extends3));
    }

    if (len === 4) {
      var _extends4;

      return _extends({}, shorthand, (_extends4 = {}, _defineProperty(_extends4, property + 'Bottom', v3), _defineProperty(_extends4, property + 'Left', v4), _extends4));
    }
  }

  return {};
}

function toStyleSheet(styleData) {
  var _parser = (0, _cssParse2.default)(styleData);

  var rules = _parser.stylesheet.rules;

  var result = {};

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = rules[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step$value = _step.value;
      var ruleType = _step$value.type;
      var selectors = _step$value.selectors;
      var declarations = _step$value.declarations;

      if (ruleType !== RULE) continue;

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = selectors[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var selector = _step3.value;

          var s = selector.replace(/\.|#/g, '');
          result[s] = result[s] || {};

          var _iteratorNormalCompletion4 = true;
          var _didIteratorError4 = false;
          var _iteratorError4 = undefined;

          try {
            for (var _iterator4 = declarations[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
              var _step4$value = _step4.value;
              var declarationType = _step4$value.type;
              var value = _step4$value.value;
              var property = _step4$value.property;

              if (declarationType !== DECLARATION) continue;

              var propertyName = ['-ios-', '-android-'].reduce(function (m, v) {
                return m.replace(v, '');
              }, property);
              var camelizedPropName = (0, _toCamelCase2.default)(propertyName);

              if (value) {
                result[s] = _extends({}, result[s], _defineProperty({}, camelizedPropName, value), normalizeValue(propertyName, value), expandProperty(propertyName, value));
              }
            }
          } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
              }
            } finally {
              if (_didIteratorError4) {
                throw _iteratorError4;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  var output = JSON.stringify(result);

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = Object.keys(styleSheetConstants)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var key = _step2.value;

      output = output.replace('"' + key + '"', styleSheetConstants[key]);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return output;
}

function transformAssets(_ref4) {
  var t = _ref4.types;

  function resolveModulePath(filename) {
    var dir = (0, _path.dirname)(filename);
    if ((0, _path.isAbsolute)(dir)) return dir;
    if (process.env.PWD) return (0, _path.resolve)(process.env.PWD, dir);
    return (0, _path.resolve)(dir);
  }

  return {
    visitor: {
      CallExpression: function CallExpression(path, _ref5) {
        var file = _ref5.file;
        var opts = _ref5.opts;

        var currentConfig = _extends({}, defaultOptions, opts);

        currentConfig.extensions = currentConfig.extensions || [];

        var _path$node = path.node;
        var calleeName = _path$node.callee.name;
        var args = _path$node.arguments;


        if (calleeName !== 'require' || !args.length || !t.isStringLiteral(args[0])) {
          return;
        }

        if (currentConfig.extensions.find(function (ext) {
          return args[0].value.endsWith(ext);
        })) {
          var _args = _slicedToArray(args, 1);

          var filePath = _args[0].value;


          if (!t.isVariableDeclarator(path.parent)) {
            throw new Error('Found empty import from ' + filePath + '.');
          }

          var from = resolveModulePath(file.opts.filename);
          var data = (0, _fs.readFileSync)((0, _path.resolve)(from, filePath), 'utf8');
          var p = toStyleSheet(data);

          path.replaceWithSourceString('require(\'react-native\').StyleSheet.create(' + p + ')');
        }
      }
    }
  };
}